package yrpg.bamp.basedata.plugin.operate;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.exception.KDBizException;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.DBServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.bos.util.HttpClientUtils;
import kd.bos.util.StringUtils;
import kd.sdk.plugin.Plugin;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 单据操作插件
 */
public class PushBeBankToMDMOpPlugin extends AbstractOperationServicePlugIn implements Plugin {


    public void onPreparePropertys(PreparePropertysEventArgs e){
        e.getFieldKeys().add("id");
        e.getFieldKeys().add("name");
        e.getFieldKeys().add("country.number");
        e.getFieldKeys().add("country.name");
        e.getFieldKeys().add("province.number");
        e.getFieldKeys().add("province.name");
        e.getFieldKeys().add("city.number");
        e.getFieldKeys().add("city.name");
        e.getFieldKeys().add("union_number");
        e.getFieldKeys().add("isoverdue");
        e.getFieldKeys().add("isfromcloud");
        e.getFieldKeys().add("provincecloud");
        e.getFieldKeys().add("citycloud");
        e.getFieldKeys().add("enable");
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        DynamicObject[] bills = e.getDataEntities();
        String url=System.getProperty("yrpg.fi.mdm.interbank.url");

        Map<String,String> header =new HashMap<>();
        header.put("Content-Type","application/json;charset=utf-8");

        SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd");


        JSONObject paramsLogin = new JSONObject();
        JSONArray masterData = new JSONArray();
        for(DynamicObject bill:bills){
            JSONObject data = new JSONObject();
            long bankid=bill.getLong("id");//id
            String name=bill.getString("name");//名称
            String countrynumber="";
            String countryname="";
            String provincenumber="";
            String provincename="";
            String citynumber="";
            String cityname="";
            DynamicObject country=bill.getDynamicObject("country");
            if(country!=null){
                countrynumber=country.getString("number");//国家编码
                countryname=country.getString("name");//国家名称
            }
            DynamicObject province=bill.getDynamicObject("province");
            if(province!=null){
                provincenumber=province.getString("number");//省份编码
                provincename=province.getString("name");//省份名称
            }
            DynamicObject city=bill.getDynamicObject("city");
            if(city!=null){
                citynumber=city.getString("number");//城市编码
                cityname=city.getString("name");//城市名称
            }
            String unionnumber=bill.getString("union_number");//联行号
            String isoverdue=bill.getString("isoverdue");//疑似过期  Y:是  N:否
            Boolean isfromcloud=bill.getBoolean("isfromcloud");//云端数据  true/false
            String provincecloud=bill.getString("provincecloud");//省份(云端)
            String citycloud=bill.getString("citycloud");//城市(云端)
            String enable=bill.getString("enable");//启用状态  0:禁用   1:可用
            if("1".equals(enable)){
                enable="启用";
            }else{
                enable="停用";
            }

            data.put("id",String.valueOf(bankid));
            data.put("bank_name",name);
            data.put("region_code",countrynumber);
            data.put("region_name",countryname);
            data.put("province_code",provincenumber);
            data.put("province_name",provincename);
            data.put("province_UnionPay",provincename);
            data.put("city_code",citynumber);
            data.put("city_name",cityname);
            data.put("Interbank_number",unionnumber);
            //data.put("Bank_category",);   银行类别名称 无
            data.put("city_UnionPay",cityname);
            data.put("Suspected_expiration",isoverdue);
            data.put("cloud_data",String.valueOf(isfromcloud));
            data.put("province_cloud",provincecloud);
            data.put("city_cloud",citycloud);
            data.put("enablestate#name",enable);

            masterData.add(data);
        }

        paramsLogin.put("masterData",masterData);

        String reponseStrLogin="";
        long pkValue= DBServiceHelper.genGlobalLongId();
        Map<String, Object> logMap = new HashMap<>();
        logMap.put("opname", "推送联行号到MDM");
        logMap.put("thirdappname", "MDM");
        logMap.put("opdesc",url);
        logMap.put("request", paramsLogin.toString());
        logMap.put("status", "true");
        try{
            reponseStrLogin= HttpClientUtils.postjson(url,header,paramsLogin.toString());
        }catch (Exception exc){
            logMap.put("response", "调用MDM推送联行号接口失败！"+exc.getMessage());
            logMap.put("status", "false");
            saveApiLog(logMap);
            throw new KDBizException("推送联行号至主数据异常！异常信息为"+exc.getMessage());
        }

        if (!StringUtils.isEmpty(reponseStrLogin)) {
            JSONObject responseJSON = JSONObject.parseObject(reponseStrLogin);
            if (Boolean.valueOf(responseJSON.get("success").toString())) {
                logMap.put("response","推送联行号至主数据成功！");
                saveApiLog(logMap);
            }else{
                logMap.put("status", "false");
                logMap.put("response", "调用MDM推送联行号接口返回异常！错误信息是"+responseJSON.get("message").toString());
                saveApiLog(logMap);
                throw new KDBizException("调用EAS录接口返回异常，错误信息是" +responseJSON.get("message").toString());
            }
        } else {
            logMap.put("status", "false");
            logMap.put("response", "接口返回值为空");
            saveApiLog(logMap);
            throw new KDBizException("调用MDM推送联行号接口返回值为空");
        }
    }

    public static void saveApiLog(Map<String,Object> logMap){
        DynamicObject apiLog = BusinessDataServiceHelper.newDynamicObject("openapi_log_data");
        //api编号
        apiLog.set("opname",logMap.get("opname"));
        //URL
        apiLog.set("opdesc",logMap.get("opdesc"));
        //请求参数
        if(logMap.get("request")!=null){
            apiLog.set("request",logMap.get("request").toString().length()>2000?logMap.get("request").toString().substring(0, 2000):logMap.get("request"));
        }else {
            apiLog.set("request","");
        }

        if (logMap.get("response")!=null){
            //响应结果
            apiLog.set("response",logMap.get("response").toString().length()>2000?logMap.get("response").toString().substring(0, 2000):logMap.get("response"));
        }else {
            apiLog.set("response","");
        }
        //调用时间
        apiLog.set("opdate",new Date());
        //调用者
        apiLog.set("username",logMap.get("username"));
        //第三方应用
        apiLog.set("thirdappname",logMap.get("thirdappname"));
        //调用状态
        apiLog.set("status",logMap.get("status"));
        //保存日志
        SaveServiceHelper.save(new DynamicObject[]{apiLog});

    }
}